!**
!* The whole program.
!*  
 program driver

   use vtkRendererMod
   use vtkRenderWindowMod
   use vtkWindowMod

   type(vtkRenderer)  :: renderer
   type(vtkRenderWindow) :: renderWindow
   type(vtkWin) :: castToWindow

   integer :: width = 100
   integer :: height = 100
   integer :: x
  
   integer :: array(100*100)

   ! create the window
   call New_F( renderer )
   call New_F( renderWindow )
   call AddRenderer( renderWindow, renderer )
   call SetSize( transfer(renderWindow,castToWindow), width, height )
   
      do i=1, height-1
        do j=1, width-1
          array(width*i+j) = j
        end do
      end do

   call solver( width, height, array, renderer, renderWindow )

   ! wait until the user is done
   read *,x

 end program driver


!**
!* The laplace solver routine.
!*  
 subroutine solver( width, height, array, renderer, renderWindow )
   
   use vtkRendererMod
   use vtkRenderWindowMod

   type(vtkRenderer)     :: renderer
   type(vtkRenderWindow)    :: renderWindow
   integer :: width, height
   integer :: array(width*height)

   real(kind(1.0D0)),dimension(width,height) :: t0
   real(kind(1.0D0)),dimension(width,height) :: t1
   real(kind(1.0D0)),dimension(width,height) :: temp
   integer :: nx,ny
   real(kind(1.0D0)) :: eps = 5.0
   real(kind(1.0D0)) :: e
   integer :: t = 0, i, j

   call update( width, height, array, renderer, renderWindow )

   nx = size(t0,1)
   ny = size(t0,2)

   t0 = 0.0
   t1 = 0.0
   
   t0(1:width,1) = 100
   t1(1:width,1) = 100

!  iterate until the squares of the change in temperature summed over all
!    interior points is less than 1.0.
   do
      e = 0.0
      
      do j=2, nx-1
        do i=2, ny-1
          t1(i,j) = (t0(i-1,j) + t0(i+1,j) + t0(i,j-1) + t0(i,j+1)) / 4
 	  e = e + abs( t1(i,j) - t0(i,j) );
        end do
      end do

!     see if we're done iterating or not
      if( e*e < eps ) then 
      	exit
      end if

!     swap the arrays
      temp = t0
      t0 = t1
      t1 = temp
   
      do i=1, height-1
        do j=1, width-1
          array(width*i+j) = t0(i,j)
        end do
      end do

      if( mod(real(t),10.0) == 0 ) then
        call update( width, height, array, renderer, renderWindow )
      endif
      
      t = t + 1;
   end do

 end subroutine solver

!**
!* Update the imageplot.
!*  
 subroutine update( width, height, array, renderer, renderWindow )

  use vtkRendererMod
  use vtkRenderWindowMod
  use vtkActor2DMod
  use vtkImageImportMod
  use vtkLookupTableMod
  use vtkImageMapToColorsMod
  use vtkImageViewerMod
  use vtkImageImportMod
  use vtkImageDataMod
  use vtkScalarsToColorsMod
  !use vtkObjectMod
  use vtkImageToImageFilterMod
  use vtkPropMod
  use vtkImageSourceMod

  type(vtkRenderer)     :: renderer
  type(vtkRenderWindow) :: renderWindow

  type(vtkImgImp) :: imageImport
  type(vtkImgMapToCols) :: imageMap
  type(vtkImgViewer) :: imageViewer
  type(vtkAct2D) :: actor
  type(vtkImageData) :: imageData
  type(vtkLupTab) :: lookupTable
  
  type(vtkImgToImgFtr) :: castToImgToImgFilter
  type(vtkProp)        :: castToProp
  type(vtkImageSource) :: castToImageSource
  type(vtkScalsToCols) :: castToScalarsToColors

  integer :: width, height
  integer :: array(width*height)

  integer :: datatype = 6 		! VTK_INT
  integer :: elemSize = 4 		! sizeof(int)

  ! create the image importer object and copy in the array
  call New_F( imageImport )
  call SetWholeExtent( imageImport, 0, width-1, 0, height-1, 0, 0 )
  call SetDataExtWholeExt( imageImport )
  call SetDataScalarType( imageImport, datatype )  
  call CopyImportVoidPointer( imageImport, array, elemSize*width*height )

  ! create a color lookup table
  call New_F( lookupTable )
  call SetTableRange( lookupTable, 0.0, real(width) )
  call SetRampToLinear( lookupTable )
  call Build( lookupTable )

  ! map the colors to the array data
  call New_F( imageMap )
  call GetOutput_F( transfer( imageImport, castToImageSource ), imageData )  
  call SetInput( transfer( imageMap, castToImgToImgFilter ), imageData )
  call SetOutputFormatToRGB( imageMap )
  call SetLookupTable( imageMap, transfer( lookupTable, castToScalarsToColors ))

  ! create an image viewer and get an actor out of it
  call New_F( imageViewer )
  call GetOutput_F( transfer( imageMap, castToImageSource ), imageData )  
  call SetInput( imageViewer, imageData )
  call SetColorWindow( imageViewer, 255.0 )
  call SetColorLevel( imageViewer, 127.0 )
  call GetActor2D_F( imageViewer, actor )
  
  ! add the actor to the screen and render it
  call AddActor( renderer, transfer( actor, castToProp ))
  call Render( renderWindow )

 end subroutine update
