
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:import href="common.xsl"/>
  <xsl:output method="text"/>



 <!--
    Root level.  This gets things going.
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>



  <!--
    Library level (is there ever more than one library?).
    -->
  <xsl:template match="library">
    <xsl:call-template name="open_file"/>
    
    <!-- this only will look to create types in the global scope, or one level deep in a structure -->
    <xsl:apply-templates select="scope/structure|scope/typealias|scope/structure/typealias|scope/structure/structure" mode="generate_strucs"/>    

    <xsl:call-template name="close_file"/>
  </xsl:template>



  <!--
    Generate the structure declarations for any structs declared in
    the C code.  We don't generate wrapper code for anything that uses
    non-public objects, so therefore we don't need to generate types
    for non-public objects.
    -->
  <xsl:template match="structure|typealias" mode="generate_strucs">

    <!-- we only generate types for public methods -->
    <xsl:if test="@access != 'prot' and @access != 'priv'">

    <xsl:text>module </xsl:text>
    <xsl:call-template name="gen_type_name_f">
      <xsl:with-param name="append_type" select="'yes'"/>
      <xsl:with-param name="type_name" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>
    
    <xsl:text>type </xsl:text>
    <xsl:call-template name="gen_type_name_f">
      <xsl:with-param name="type_name" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:text>integer x</xsl:text>
    <xsl:value-of select="$newline"/>

    <xsl:text>end type </xsl:text>
    <xsl:call-template name="gen_type_name_f">
      <xsl:with-param name="type_name" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$newline"/>

    <xsl:text>end module </xsl:text>
    <xsl:call-template name="gen_type_name_f">
      <xsl:with-param name="append_type" select="'yes'"/>
      <xsl:with-param name="type_name" select="@name"/>
    </xsl:call-template>
    <xsl:value-of select="$dblnewline"/>

    </xsl:if>

  </xsl:template>



  <!--
    Print file header information.
    -->
  <xsl:template name="open_file">

    <xsl:text>!</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>! fortran calling c - this file is automatically generated</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:text>!</xsl:text>
    <xsl:value-of select="$dblnewline"/>

  </xsl:template>



  <!--
    Finish up.
    -->
  <xsl:template name="close_file">
    <xsl:value-of select="$newline"/>
  </xsl:template>



</xsl:stylesheet>
