/** LANL:license
 * -------------------------------------------------------------------------
 * This SOFTWARE has been authored by an employee or employees of the
 * University of California, operator of the Los Alamos National Laboratory
 * under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 * The U.S. Government has rights to use, reproduce, and distribute this
 * SOFTWARE.  The public may copy, distribute, prepare derivative works and
 * publicly display this SOFTWARE without charge, provided that this Notice
 * and any statement of authorship are reproduced on all copies.  Neither
 * the Government nor the University makes any warranty, express or implied,
 * or assumes any liability or responsibility for the use of this SOFTWARE.
 * If SOFTWARE is modified to produce derivative works, such modified
 * SOFTWARE should be clearly marked, so as not to confuse it with the
 * version available from LANL.
 * -------------------------------------------------------------------------
 * LANL:license
 * -------------------------------------------------------------------------
 */
/**
 * ByteArrayClassLoader
 *
 * Simple class to provide class loading facilities from byte arrays.
 * Given a byte array containing the contents of a .class file, we can
 * load the class from there.  This is useful if receiving classes over
 * RMI or socket connections.
 */
public class ByteArrayClassLoader extends ClassLoader {
    private byte[] classData = null;

    public ByteArrayClassLoader(byte[] classData) {
	this.classData = classData;
    }

    public Class loadClass(String name) 
	throws ClassNotFoundException {
	return loadClass(name, true);
    }

    public Class loadClass(String name, boolean resolve) 
	throws ClassNotFoundException {

	try {
            //
            // try to see if the class is already loaded
            //
	    Class c = findLoadedClass(name);

            // 
            // not loaded, so now see if its a system provided class
            //
	    if (c==null) {
		try { c = findSystemClass(name); }
		catch (Exception e) { }
	    }

            //
            // so far not a system class and not a loaded class, so
            // try to load it out of the classData byte array if it
            // isn't null.
            //
	    if (c==null) {
                if (classData != null) {
                    c = defineClass(name, classData, 0, classData.length);
                } else {
                    throw new ClassNotFoundException("ByteArrayClassLoader:"+
                                                     " cannot load from a"+
                                                     " null byte array.");
                }
            }

            //
            // resolve?
            //
	    if (resolve)
		resolveClass(c);
	    
            //
            // return the class we loaded
            //
	    return c;
	} catch (Exception e) {
	    throw new ClassNotFoundException(e.toString());
	}
    }
}
