<!-- LANL:license
 ...........................................................................
 - This SOFTWARE has been authored by an employee or employees of the
 - University of California, operator of the Los Alamos National Laboratory
 - under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 - The U.S. Government has rights to use, reproduce, and distribute this
 - SOFTWARE.  The public may copy, distribute, prepare derivative works and
 - publicly display this SOFTWARE without charge, provided that this Notice
 - and any statement of authorship are reproduced on all copies.  Neither
 - the Government nor the University makes any warranty, express or implied,
 - or assumes any liability or responsibility for the use of this SOFTWARE.
 - If SOFTWARE is modified to produce derivative works, such modified
 - SOFTWARE should be clearly marked, so as not to confuse it with the
 - version available from LANL.
 ...........................................................................
 - LANL:license
 ...........................................................................
 -->

<!--
  - testverify.c.xsl: creates test routines for nullifyArrayDesc and 
  -    verifyArrayDesc procedures.
  -
  - Output should be directed to the file, testverify.c
  -->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

  <xsl:output method="text"/>

  <xsl:variable name="newline">
<xsl:text>
</xsl:text>
  </xsl:variable>

 <!--
    - root level
    -->
  <xsl:template match="/">
    <xsl:apply-templates select="library"/>
  </xsl:template>


  <!--
    - library level
    -->
  <xsl:template match="library">
    <xsl:call-template name="openFile"/>

    <xsl:text>#ifdef F90_SYM_CASE_LOWER</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-lowercase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_LOWER */</xsl:text>
    <xsl:value-of select="concat($newline, $newline, $newline)"/>

    <xsl:text>#ifdef F90_SYM_CASE_MIXED</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-mixedcase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_MIXED */</xsl:text>
    <xsl:value-of select="concat($newline, $newline, $newline)"/>

    <xsl:text>#ifdef F90_SYM_CASE_UPPER</xsl:text>
    <xsl:value-of select="$newline"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="symbol-uppercase">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:text>#endif /* F90_SYM_CASE_UPPER */</xsl:text>
    <xsl:value-of select="concat($newline, $newline)"/>

    <xsl:value-of select="$newline"/>
    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:text>int testArray</xsl:text>
        <xsl:value-of select="concat($rank, 'D', @name, '(void*);')"/>
        <xsl:value-of select="$newline"/>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:value-of select="concat($newline, $newline)"/>

    <xsl:for-each select="type">
      <xsl:if test="@ctype">
        <xsl:call-template name="define-initArray">
          <xsl:with-param name="type" select="@name"/>
          <xsl:with-param name="ctype" select="@ctype"/>
        </xsl:call-template>
      </xsl:if>
    </xsl:for-each>

    <xsl:call-template name="openMain"/>

    <xsl:for-each select="dim">
      <xsl:variable name="rank" select="@rank"/>
      <xsl:for-each select="../type">
        <xsl:call-template name="call-create-destroy">
          <xsl:with-param name="rank" select="$rank"/>
          <xsl:with-param name="type" select="@name"/>
          <xsl:with-param name="ctype" select="@ctype"/>
        </xsl:call-template>
      </xsl:for-each>
    </xsl:for-each>

    <xsl:call-template name="closeFile"/>
  </xsl:template>


  <!--
    - define-initArray
    -->
  <xsl:template name="define-initArray">
    <xsl:param name="type"/>
    <xsl:param name="ctype"/>
    <xsl:variable name="name" select="concat('initArray', $type)"/>

    <xsl:value-of select="concat('void ', $name)"/>
    <xsl:text>(void* dv, int rank, int* lb, int* ub)</xsl:text>
    <xsl:value-of select="concat(' {', $newline, '  ', $ctype, '* a;')"/>
    <xsl:text>
  int i;
  int size = 1;
  for (i = 0; i &lt; rank; i++) size *= 1 + ub[i] - lb[i];
  </xsl:text>
    <xsl:value-of select="concat('a = (', $ctype, '*) ')"/>
    <xsl:text>getArrayBaseAddress(dv, rank);
  for (i = 0; i &lt; size; i++) *a++ = -rank;
</xsl:text>
    <xsl:value-of select="concat('}', $newline, $newline)"/>

  </xsl:template>


  <!--
    - openMain
    -->
  <xsl:template name="openMain">
    <xsl:text>
int F90_MAIN(int argc, char* argv[])
{
  void* dv;
  int nErrors = 0;
  F90_CompilerCharacteristics cc;

  long lb[] = {2, 3, 4, 5, 6, 7, 8};
  long ub[] = {5, 7, 9, 11, 13, 15, 17};

  F90_SetCompilerCharacteristics(&amp;cc, FORTRAN_COMPILER);


</xsl:text>
  </xsl:template>


  <!--
    - call-create-destroy
    -->
  <xsl:template name="call-create-destroy">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:param name="ctype"/>

    <xsl:value-of select="concat('  F90_CreateArray', $rank, 'D', $type)"/>
    <xsl:value-of select="concat('(&amp;dv, lb, ub);', $newline)"/>

    <xsl:text>  if ( cc.verifyArrayDesc(dv,</xsl:text>
    <xsl:value-of select="$rank"/> <xsl:text>) ) {
    nErrors += 1;
    printf("ERROR, verifyArrayDesc failed (</xsl:text>
    <xsl:value-of select="concat($rank, 'D', $type)"/> <xsl:text>)\n");
  }
  cc.nullifyArrayDesc(dv,</xsl:text> <xsl:value-of select="$rank"/> <xsl:text>);

  if ( ! cc.verifyArrayDesc(dv,</xsl:text> <xsl:value-of select="$rank"/>
    <xsl:text>) ) {
    nErrors += 1;
    printf("ERROR, verifyArrayDesc should have failed (</xsl:text>
    <xsl:value-of select="concat($rank, 'D', $type)"/> <xsl:text>)\n");
  }
  if ( testArray</xsl:text> <xsl:value-of select="concat($rank, 'D', $type)"/>
    <xsl:text>(dv) != 1 ) {
    nErrors += 1;
    printf("ERROR, testArray should have failed (</xsl:text>
    <xsl:value-of select="concat($rank, 'D', $type)"/> <xsl:text>)\n");
  }


</xsl:text>

  </xsl:template>


  <!--
    - symbol-lowercase
    -->
  <xsl:template name="symbol-lowercase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat('testArray', $rank, 'D', $type)"/>
    <xsl:variable name="name_lower">
      <xsl:call-template name="lower-case">
        <xsl:with-param name="symbol" select="$name"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:value-of select="concat('#  define ', $name,
                             '  F90_SYMBOL( ', $name_lower, ' )', $newline)"/>

  </xsl:template>


  <!--
    - symbol-mixedcase
    -->
  <xsl:template name="symbol-mixedcase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat('testArray', $rank, 'D', $type)"/>

    <xsl:value-of select="concat('#  define ', $name,
                             '  F90_SYMBOL( ', $name, ' )', $newline)"/>

  </xsl:template>


  <!--
    - symbol-uppercase
    -->
  <xsl:template name="symbol-uppercase">
    <xsl:param name="rank"/>
    <xsl:param name="type"/>
    <xsl:variable name="name" select="concat('testArray', $rank, 'D', $type)"/>
    <xsl:variable name="name_upper">
      <xsl:call-template name="upper-case">
        <xsl:with-param name="symbol" select="$name"/>
      </xsl:call-template>
    </xsl:variable>

    <xsl:value-of select="concat('#  define ', $name,
                             '  F90_SYMBOL( ', $name_upper, ' )', $newline)"/>

  </xsl:template>


  <!--
    - openFile
    -->
  <xsl:template name="openFile">

<xsl:text>/** LANL:license
 * -------------------------------------------------------------------------
 * This SOFTWARE has been authored by an employee or employees of the
 * University of California, operator of the Los Alamos National Laboratory
 * under Contract No. W-7405-ENG-36 with the U.S. Department of Energy.
 * The U.S. Government has rights to use, reproduce, and distribute this
 * SOFTWARE.  The public may copy, distribute, prepare derivative works and
 * publicly display this SOFTWARE without charge, provided that this Notice
 * and any statement of authorship are reproduced on all copies.  Neither
 * the Government nor the University makes any warranty, express or implied,
 * or assumes any liability or responsibility for the use of this SOFTWARE.
 * If SOFTWARE is modified to produce derivative works, such modified
 * SOFTWARE should be clearly marked, so as not to confuse it with the
 * version available from LANL.
 * -------------------------------------------------------------------------
 * LANL:license
 * -------------------------------------------------------------------------
 *
 * testverify.c - this file is automatically created
 *
 * This file tests the verify/nullifyArrayDesc functions
 *
 */

#include &lt;F90CreateArray.h&gt;

#ifdef F90_ABSOFT
#  define getArrayBaseAddress getArrayBaseAddress_Absoft
#endif

#ifdef F90_ALPHA
#  define getArrayBaseAddress getArrayBaseAddress_Alpha
#endif

#ifdef F90_CRAY
#  define getArrayBaseAddress getArrayBaseAddress_Cray
#endif

#ifdef F90_GNU
#  define getArrayBaseAddress getArrayBaseAddress_GNU
#endif

#ifdef F90_G95
#  define getArrayBaseAddress getArrayBaseAddress_G95
#endif

#ifdef F90_IBMXL
#  define getArrayBaseAddress getArrayBaseAddress_IBMXL
#endif

#ifdef F90_INTEL
#  define getArrayBaseAddress getArrayBaseAddress_Intel
#endif

#ifdef F90_INTEL_7
#  define getArrayBaseAddress getArrayBaseAddress_Intel_7
#endif

#ifdef F90_LAHEY
#  define getArrayBaseAddress getArrayBaseAddress_Lahey
#endif

#ifdef F90_MIPSPRO
#  define getArrayBaseAddress getArrayBaseAddress_MIPSpro
#endif

#ifdef F90_NAG
#  define getArrayBaseAddress getArrayBaseAddress_NAG
#endif

#ifdef F90_PGI
#  define getArrayBaseAddress getArrayBaseAddress_PGI
#endif

#ifdef F90_SUNWSPRO
#  define getArrayBaseAddress getArrayBaseAddress_SUNWspro
#endif

</xsl:text>

  </xsl:template>


  <!--
    - closeFile: finish up
    -->
  <xsl:template name="closeFile">
    <xsl:value-of select="concat('  return nErrors;', $newline, '}', $newline)"/>
  </xsl:template>


  <!--
    - upper-case
    -->
  <xsl:template name="upper-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'abcdefghijklmnopqrstuvwxyz',
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:template>


  <!--
    - lower-case
    -->
  <xsl:template name="lower-case">
    <xsl:param name="symbol" select="unknown_symbol"/>

    <xsl:value-of select="translate($symbol,
                             'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
                             'abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:template>


</xsl:stylesheet>
