module Complex_type
  type Complex_t
    real :: r
    real :: i
  end type Complex_t
end module Complex_type


function test_int_pointer(ip)
  implicit none
  integer, pointer :: ip
  integer :: test_int_pointer

  test_int_pointer = 0

  if (.not. associated(ip)) then
    print *, "ERROR, test_int_pointer has unassociated pointer"
    test_int_pointer = 1
    return
  end if

  if (ip .ne. 33) then
    print *, "ERROR, test_int_pointer: pointer value not equal 33 = ", ip
    test_int_pointer = 1
    return
  end if

  ip = 4

end function test_int_pointer


function test_float_pointer(xp)
  implicit none
  real, pointer :: xp
  integer :: test_float_pointer

  test_float_pointer = 0

  if (.not. associated(xp)) then
    print *, "ERROR, test_float_pointer has unassociated pointer"
    test_float_pointer = 1
    return
  end if

  if (xp .ne. 33.3) then
    print *, "ERROR, test_float_pointer: pointer value not equal 33.3 = ", xp
    test_float_pointer = 1
    return
  end if

  xp = 99.9

end function test_float_pointer


function test_complex_pointer(cp)
  implicit none
  complex, pointer :: cp
  integer :: test_complex_pointer

  test_complex_pointer = 0

  if (.not. associated(cp)) then
    print *, "ERROR, test_complex_pointer has unassociated pointer"
    test_complex_pointer = 1
    return
  end if

  if (real(cp) .ne. -33.3 .and. aimag(cp) .ne. 99.9) then
    print *, "ERROR, test_complex_pointer: pointer value not equal (-33.3,99.0) = ", cp
    test_complex_pointer = 1
    return
  end if

  cp = (4.0, 5.0)

end function test_complex_pointer


function test_derived_pointer(dp)
  use Complex_type
  implicit none
  type(Complex_t), pointer :: dp
  integer :: test_derived_pointer

  test_derived_pointer = 0

  if (.not. associated(dp)) then
    print *, "ERROR, test_derived_pointer has unassociated pointer"
    test_derived_pointer = 1
    return
  end if

  if (dp%r .ne. 4.0 .and. dp%i .ne. 5.0) then
    print *, "ERROR, test_derived_pointer: pointer value not equal (4,5) = ", dp
    test_derived_pointer = 1
    return
  end if

  dp%r = 5.0
  dp%i = 4.0

end function test_derived_pointer
